<!-- This file is being drafted here ahead of being merged into the guidebook. -->

# OpenAPI client generation

This document describes the use of the OpenAPI generator template. The OpenAPI generator template is
intended to be included in the "common" pipeline and so will not add any jobs to the pipeline unless
specifically triggered.

## Basic use

Add a file named `openapi.yaml` to the root of your repository with the OpenAPI schema in it. This
will trigger a run of the OpenAPI client generator. No additional CI configuration is needed.

If you schema is stored in a different file, the `OPENAPI_GENERATOR_SCHEMA_PATH` CI variable may be
set to override the schema location. The variable should be set to a path relative to the repository
root which points to the schema file.

## Generating the schema in CI

If you generate your OpenAPI schema dynamically, you need to explicitly enable OpenAPI client
generation by setting the `OPENAPI_GENERATOR_ENABLED` variable. Additionally you'll need to override
the `openapi:schema` job to generate your schema and write it to the path stored in the
`OPENAPI_GENERATOR_SCHEMA_ARTIFACT_PATH` variable.

For example:

```yaml
# .gitlab-ci.yml

variables:
  # We need to explicitly enable the OpenAPI generation since we don't have an openapi.yaml file in
  # the root of our repository which would otherwise signal that.
  OPENAPI_GENERATOR_ENABLED: "1"

# We do not ship the OpenAPI schema in the repository and so the openapi:schema job needs to be
# overridden to use our schema generation command.
openapi:schema:
  script:
    - my-schema-generator --output=$OPENAPI_GENERATOR_SCHEMA_ARTIFACT_PATH
```

Not that the existing `openapi:schema` job's `before_script` ensures that the directory containing
that path exists and so you do not need a `mkdir` or equivalent.

## CI variables

The following variables can be set to change the behaviour of the CI template.

|Variable|Default|Description|
|-|-|-|
|`OPENAPI_GENERATOR_ENABLED`|*unset*|Set to non-empty value to *enable* OpenAPI client generation if not otherwise automatically triggered.|
|`OPENAPI_GENERATOR_DISABLED`|*unset*|Set to non-empty value to *disable* OpenAPI client generation if automatically triggered when it shouldn't be.|
|`OPENAPI_GENERATOR_SCHEMA_PATH`|`openapi.yaml`|Location of static OpenAPI schema file within the repository.|
|`OPENAPI_GENERATOR_ARTIFACT_DIR`|`openapi`|Directory in which generated artifacts are placed.|
