import random

import factory

from .models import Account, AccountDetails


class AccountFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Account

    date_of_birth = factory.Faker("date_between", start_date="-100y", end_date="-10y")
    last_name = factory.Faker("last_name")

    @factory.lazy_attribute
    def crsid(self):
        # Choose 'abcdejmnrst' as the first letter for CRSid because these are the most
        # commonly used initials in names, which helps minimize the risk of infinite loops
        # in the name matching process within the AccountDetailsFactory.
        return (
            f"{random.choice('abcdejmnrst')}{self.last_name[:1].lower()}"
            f"{random.randint(199, 99999)}"
        )

    @factory.lazy_attribute
    def code(self):
        code_type = random.choice(["staff", "postgraduate", "pgce", "undergraduate"])
        faker = factory.Faker._get_faker()
        if code_type == "staff":
            return faker.bothify("???###?##").lower()
        elif code_type == "postgraduate":
            return faker.numerify(text="3########")
        elif code_type == "pgce":
            return faker.numerify(text="#######")
        elif code_type == "undergraduate":
            return faker.numerify(text="##########")


class AccountDetailsFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AccountDetails

    account = factory.SubFactory(AccountFactory)
    college = factory.Faker("company")
    affiliation = factory.Faker("company")
    terms_accepted = False

    @factory.lazy_attribute
    def name(self):
        # Generate first name based on a letter from crsid
        faker = factory.Faker._get_faker()
        first_letter = self.account.crsid[0].upper()
        first_name = faker.first_name()

        attempts = 0
        max_attempts = 20
        while not first_name.startswith(first_letter) and attempts < max_attempts:
            first_name = faker.first_name()
            attempts += 1

        last_name = self.account.last_name
        return f"{first_name} {last_name}"
