import random

import pytest
from django.db import IntegrityError

from activate_account.factories import AccountFactory
from activate_account.models import Account

pytestmark = pytest.mark.django_db


def test_account_uniqueness_together(account):
    with pytest.raises(IntegrityError):
        AccountFactory(
            last_name=account.last_name, date_of_birth=account.date_of_birth, code=account.code
        )


def test_account_unique_together_case_insensitive(account):
    last_name_upper = account.last_name.upper()
    assert account.last_name != last_name_upper  # Sanity check

    with pytest.raises(IntegrityError):
        AccountFactory(
            last_name=last_name_upper,
            date_of_birth=account.date_of_birth,
            code=account.code,
        )


def test_account_individiual_uniqueness(account):
    for attribute in ["last_name", "date_of_birth", "code"]:
        AccountFactory(**{attribute: getattr(account, attribute)})


def test_account_separate_uniqueness():
    n_accounts = random.randrange(10, 21)

    for _ in range(n_accounts):
        AccountFactory()

    assert Account.objects.count() == n_accounts


def test_account_crsid_case():
    factory_account = AccountFactory(crsid="ABC123")
    account = Account.objects.get(pk="abc123")
    factory_account.refresh_from_db()
    assert account == factory_account

    # Check that creating a CRSId with different casing raises an integrity error
    with pytest.raises(IntegrityError):
        factory_account = AccountFactory(crsid="aBc123")
