"""
Views implementing the API endpoints.

"""

from drf_spectacular.utils import OpenApiResponse, extend_schema
from rest_framework import generics

from api.v1alpha.serializers import AccountSerializer, MethodNotAllowedErrorSerializer


class AccountDetailsView(generics.RetrieveUpdateAPIView):
    serializer_class = AccountSerializer
    http_method_names = ["get", "patch", "options", "head"]

    def get_object(self):
        return self.request.user

    @extend_schema(
        tags=["account management"],
        summary="Retrieve Account Details",
        description="Retrieves the account details of the currently logged-in user.",
        responses={
            200: OpenApiResponse(
                response=AccountSerializer,
                description="Successful retrieval of account details.",
            ),
        },
        methods=["GET"],
        operation_id="getAccount",
    )
    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)

    @extend_schema(
        tags=["account management"],
        summary="Update Account Details",
        description="Updates the account details of the currently logged-in user.",
        responses={
            200: OpenApiResponse(
                response=AccountSerializer,
                description="Successful update of account details.",
            ),
            204: OpenApiResponse(
                description="Successful update with no content to return "
                "when updating certain fields."
            ),
            405: OpenApiResponse(
                response=MethodNotAllowedErrorSerializer,
                description="Attempted to perform an unsupported operation.",
            ),
        },
        methods=["PATCH"],
        operation_id="patchAccount",
    )
    def patch(self, request, *args, **kwargs):
        return super().patch(request, *args, **kwargs)
